@@ -233,10 +233,10 @@ class Samples(Sampling):
                     nuv * yu * kv,
                 ],
                 [
-                    jpx / yx * y + ipy / iy * jx / yx * x,
-                    ypx / yx * y + ipy / iy * x,
-                    nxy * jx / yx,
-                    nxy,
+                    jpx * y + ipy / iy * jx * x,
+                    ypx * y + ipy / iy * yx * x,
+                    nxy * jx,
+                    nxy * yx,
                 ],
                 [
                     hew * nuv * ju * kv,
@@ -245,10 +245,10 @@ class Samples(Sampling):
                     ee * ypu * kv * v + kpv * yu * u,
                 ],
                 [
-                    hew * nxy * jx / yx,
-                    hew * nxy,
-                    ee * jpx / yx * y + ipy / iy * jx / yx * x,
-                    ee * ypx / yx * y + ipy / iy * x,
+                    hew * nxy * jx,
+                    hew * nxy * yx,
+                    ee * jpx * y + ipy / iy * jx * x,
+                    ee * ypx * y + ipy / iy * yx * x,
                 ],
             ]
         )
@@ -295,24 +295,17 @@ class Samples(Sampling):
             [
                 [0, 0, 0, 0],
                 [
-                    (
-                        (jppx / yx - jpx * ypx / yx ** 2) * y
-                        + ipy / iy * ((jpx / yx - jx * ypx / yx ** 2) * x + jx / yx)
-                    ),
-                    (yppx / yx - ypx ** 2 / yx ** 2) * y + ipy / iy,
-                    dnxy_dx * jx / yx + nxy * jpx / yx - nxy * jx * ypx / yx ** 2,
-                    dnxy_dx,
+                    jppx * y + ipy / iy * (jpx * x + jx) + 1j * sign_u * a[1, 0],
+                    yppx * y + ipy / iy * (ypx * x + yx) + 1j * sign_u * a[1, 1],
+                    dnxy_dx * jx + nxy * jpx + 1j * sign_u * a[1, 2],
+                    dnxy_dx * yx + nxy * ypx + 1j * sign_u * a[1, 3],
                 ],
                 [0, 0, 0, 0],
                 [
-                    hew
-                    * (dnxy_dx * jx / yx + nxy * jpx / yx - nxy * jx * ypx / yx ** 2),
-                    hew * dnxy_dx,
-                    (
-                        ee * (jppx / yx - jpx * ypx / yx ** 2) * y
-                        + ipy / iy * ((jpx / yx - jx * ypx / yx ** 2) * x + jx / yx)
-                    ),
-                    ee * (yppx / yx - ypx ** 2 / yx ** 2) * y + ipy / iy,
+                    hew * (dnxy_dx * jx + nxy * jpx) + 1j * sign_u * a[3, 0],
+                    hew * (dnxy_dx * yx + nxy * ypx) + 1j * sign_u * a[3, 1],
+                    ee * jppx * y + ipy / iy * (jpx * x + jx) + 1j * sign_u * a[3, 2],
+                    ee * yppx * y + ipy / iy * (ypx * x + yx) + 1j * sign_u * a[3, 3],
                 ],
             ]
         )
@@ -321,17 +314,17 @@ class Samples(Sampling):
             [
                 [0, 0, 0, 0],
                 [
-                    jpx / yx + (ippy / iy - ipy ** 2 / iy ** 2) * jx / yx * x,
-                    ypx / yx + (ippy / iy - ipy ** 2 / iy ** 2) * x,
-                    dnxy_dy * jx / yx,
-                    dnxy_dy,
+                    jpx + (ippy / iy - ipy ** 2 / iy ** 2) * jx * x,
+                    ypx + (ippy / iy - ipy ** 2 / iy ** 2) * yx * x,
+                    dnxy_dy * jx,
+                    dnxy_dy * yx,
                 ],
                 [0, 0, 0, 0],
                 [
-                    hew * dnxy_dy * jx / yx,
-                    hew * dnxy_dy,
-                    ee * jpx / yx + (ippy / iy - ipy ** 2 / iy ** 2) * jx / yx * x,
-                    ee * ypx / yx + (ippy / iy - ipy ** 2 / iy ** 2) * x,
+                    hew * dnxy_dy * jx,
+                    hew * dnxy_dy * yx,
+                    ee * jpx + (ippy / iy - ipy ** 2 / iy ** 2) * jx * x,
+                    ee * ypx + (ippy / iy - ipy ** 2 / iy ** 2) * yx * x,
                 ],
             ]
         )
@@ -346,14 +339,14 @@ class Samples(Sampling):
                     [0, 0, 0, 0],
                     [0, 0, 0, 0],
                     [
-                        ee / w2 * nuv * ju * kv,
-                        ee / w2 * nuv * yu * kv,
+                        1.0 / (e2 * w2) * nuv * ju * kv,
+                        1.0 / (e2 * w2) * nuv * yu * kv,
                         0,
                         0,
                     ],
                     [
-                        ee / w2 * nxy * jx / yx,
-                        ee / w2 * nxy,
+                        1.0 / (e2 * w2) * nxy * jx,
+                        1.0 / (e2 * w2) * nxy * yx,
                         0,
                         0,
                     ],
@@ -448,36 +441,19 @@ class Samples(Sampling):
                 pol = "E"
                 if n == 0 and i == num_m + 1:
                     roots = []
+            args = (w, pol, n, e1, e2, self.r, self.ri, np.array(roots, dtype=complex))
             result = root(
                 coax_utils.eig_eq_with_jac,
-                # coax_utils.eig_eq,
                 np.array([xi.real, xi.imag]),
-                # result.x,
-                args=(
-                    w,
-                    pol,
-                    n,
-                    e1,
-                    e2,
-                    self.r,
-                    self.ri,
-                    np.array(roots, dtype=complex),
-                ),
-                # method="krylov",
+                args=args,
                 jac=True,
-                # jac=False,
                 method="hybr",
                 options={"col_deriv": True},
             )
             x = result.x[0] + result.x[1] * 1j
-            # v = self.v(x, w, e2)
             if result.success:
                 roots.append(x)
             success.append(result.success)
-            # if v.real > 0.0:
-            #    success.append(result.success)
-            # else:
-            #     success.append(False)
             vals.append(x)
         return np.array(vals), np.array(success)
 
@@ -586,7 +562,7 @@ class Samples(Sampling):
             xs = self.beta2_pec(w, n)
             return xs, np.ones_like(xs, dtype=bool)
 
-        def func(_w, _x0s):
+        def func(_w, _x0s) -> tuple[np.ndarray, np.ndarray]:
             num_m = self.params["num_m"]
             roots: list[complex] = []
             vals = []
@@ -598,25 +574,28 @@ class Samples(Sampling):
                     pol = "E"
                     if n == 0 and i == num_m + 1:
                         roots = []
+                args = (
+                    _w,
+                    pol,
+                    n,
+                    self.fill(_w),
+                    self.clad(_w),
+                    self.r,
+                    self.ri,
+                    np.array(roots, dtype=complex),
+                )
                 result = root(
                     coax_utils.eig_eq_with_jac,
                     np.array([_x0.real, _x0.imag]),
-                    args=(
-                        _w,
-                        pol,
-                        n,
-                        self.fill(_w),
-                        self.clad(_w),
-                        self.r,
-                        self.ri,
-                        np.array(roots, dtype=complex),
-                    ),
+                    args=args,
                     jac=True,
                     method="hybr",
                     options={"col_deriv": True},
                 )
                 x = result.x[0] + 1j * result.x[1]
                 success = result.success
+                if not success and np.hypot(*result.fun) < 1e-8:
+                    success = True
                 if success:
                     roots.append(x)
                 vals.append(x)
